import { superValidate } from "sveltekit-superforms";
import { zod } from "sveltekit-superforms/adapters";
import { schema } from "./schema";
import { fail, redirect } from "@sveltejs/kit";
import { getRandomItems } from "$lib/utils";
import { GameStage, Game, type GameState } from "$lib/game";
import { getPromotionRange } from "$lib/data";
import { client } from "$lib/graphql";
import { USER_DETAILS_QUERY } from "$lib/graphql/queries";
import { handleGqlError } from "$lib/graphql/error";

type Option = {
  value: string;
  label: string;
};

async function next(state: GameState) {
  const all = await getPromotionRange(state.year, state.maxYear);
  const previous = new Set(state.history);
  const available = all.difference(previous);

  state.options = getRandomItems(Array.from(available), 4);
  state.solution = getRandomItems([...state.options], 1)[0];
  state.stage = GameStage.PLAYING;
  state.step++;
}

export async function load(event) {
  const game = new Game(event.cookies);

  switch (game.state.stage) {
    case GameStage.NEW:
      redirect(303, "/quiz/new");
      break;
    case GameStage.GAME_OVER:
      redirect(303, "/quiz/game-over");
      break;
    case GameStage.NEXT:
      await next(game.state);
      game.save();
      break;
  }

  const details = await client.query(USER_DETAILS_QUERY, {
    idList: game.state.options,
  });
  handleGqlError(details);

  const users = details.data!.page.nodes ?? [];

  const options: Option[] =
    users.map((node) => ({ label: node.nickname!, value: node.id })) ?? [];

  const photo = users.find((node) => node.id === game.state.solution)?.photo;

  const form = await superValidate(zod(schema));

  const countdown = Math.round(
    Math.max(0, MAX_POINTS + SERVER_DELAY_MAX - elapsed(game.state.timestamp)),
  );

  return {
    form,
    options,
    score: game.state.score,
    step: game.state.step,
    photo,
    label: game.state.label,
    countdown,
  };
}

const MAX_POINTS = 10;
const MIN_POINTS = 3;
// Durée maximale pour obtenir le score maximal
const SERVER_DELAY_MAX = 1;

function elapsed(timestamp: number): number {
  return (Date.now() - timestamp) / 1000;
}

function computePoints(timestamp: number): number {
  return Math.ceil(
    Math.max(
      MIN_POINTS,
      Math.min(SERVER_DELAY_MAX - elapsed(timestamp), 0) + MAX_POINTS,
    ),
  );
}

export const actions = {
  async results(event) {
    const form = await superValidate(event, zod(schema));

    if (!form.valid) {
      return fail(400, { form });
    }

    const state = new Game(event.cookies);

    const points =
      form.data.choice === state.state.solution
        ? computePoints(state.state.timestamp)
        : 0;
    state.state.score += points;
    state.state.history.push(state.state.solution);
    state.save();

    return {
      form,
      solution: state.state.solution,
      points,
    };
  },

  async next(event) {
    const form = await superValidate(zod(schema));

    const game = new Game(event.cookies);

    if (game.state.step >= 10) {
      game.state.stage = GameStage.GAME_OVER;
    } else {
      game.state.stage = GameStage.NEXT;
    }
    game.save();

    return { form };
  },

  async quit(event) {
    const game = new Game(event.cookies);

    game.reset();

    event.cookies.delete("color", { path: "/" });
  },
};
