import { superValidate } from "sveltekit-superforms";
import { zod } from "sveltekit-superforms/adapters";
import { schema } from "./schema";
import { fail, redirect } from "@sveltejs/kit";
import { Game, GameStage } from "$lib/game";
import { getPromotion } from "$lib/data";
import images from "./images";
import { sum } from "$lib/utils";
import { client } from "$lib/graphql";
import { INTE_QUERY } from "$lib/graphql/queries";
import { handleGqlError } from "$lib/graphql/error";

type BaseLevel = {
  year: number;
  maxYear?: number;
  name: string;
  image: string;
};

type Level = BaseLevel & {
  size: number;
  disabled: boolean;
};

const VIIEUX_YEAR = 20;

const MINIMUM_SIZE = 15;

export async function load(event) {
  const game = new Game(event.cookies);

  if (game.state.stage !== GameStage.NEW) redirect(303, "/quiz");

  const globals = await client.query(INTE_QUERY, {});
  handleGqlError(globals);

  const { integration } = globals.data!.globalsValues;
  const zeroA = integration ? 0 : 1;

  const baseLevels: BaseLevel[] = [
    {
      year: zeroA,
      name: `${zeroA}A`,
      image: images.stroller,
    },
    {
      year: 2,
      name: "2A",
      image: images.running,
    },
    {
      year: 3,
      name: "3A",
      image: images.stickFighting,
    },
    {
      year: zeroA,
      maxYear: 3,
      name: `${zeroA}-3A`,
      image: images.conferenceCall,
    },
    {
      year: 4,
      name: "4A",
      image: images.certificate,
    },
    {
      year: 5,
      name: "5A",
      image: images.wallet,
    },
    {
      year: 4,
      maxYear: VIIEUX_YEAR,
      name: "Viieux",
      image: images.elderlyPerson,
    },
    {
      year: zeroA,
      maxYear: VIIEUX_YEAR,
      name: "IIEns",
      image: images.crowd,
    },
  ];

  const promoSizes = await Promise.all(
    Array.from(new Array(VIIEUX_YEAR + 1), (_, i) =>
      getPromotion(i).then((promo) => promo.size),
    ),
  );

  const levels: Level[] = baseLevels.map((level) => {
    const end = (level.maxYear ?? level.year) + 1;
    const size = promoSizes.slice(level.year, end).reduce(sum, 0);
    return {
      size,
      disabled: size < MINIMUM_SIZE,
      ...level,
    };
  });

  const form = await superValidate(zod(schema));
  return { form, levels };
}

export const actions = {
  async default(event) {
    const form = await superValidate(event, zod(schema));

    if (!form.valid) {
      return fail(400, { form });
    }

    const promotion = await getPromotion(form.data.year);
    if (promotion.size < MINIMUM_SIZE) {
      return fail(400, { form });
    }

    const state = new Game(event.cookies);

    state.state.year = form.data.year;
    state.state.label = form.data.label;
    state.state.maxYear = form.data.maxYear;
    state.state.stage = GameStage.NEXT;

    state.save();

    redirect(303, "/quiz");
  },
};
